// "The two-loop helicity amplitudes for qqb' -> V1V2 -> 4 leptons"
// T. Gehrmann, A. von Manteuffel, L. Tancredi
//
// qqvvamp package
//
// qqvvampprec: compute amplitude coefficients E with precision control (recommended)


#ifndef QQVVAMPPREC_H
#define QQVVAMPPREC_H

#include "qqvvamp.h"
#include <map>

// The code below assumes double and quad precision to be available.
// If required also other types could be used.
#if defined(DOUBLE_PRECISION) && defined(QUAD_PRECISION)

// amplitude tensor coefficients for q qbar' -> V V' with numerical precision control
/* The idea is to compare results obtained with different precision settings
 * and increase the precision used in the evaluation until results are
 * consistent within a user-defined maximal uncertainty. */
class qqvvampprec {
public:
  // construct instance:
  /* Nf is number of flavors in closed loops
   * ir the IR subtraction scheme to be used for the result (see qqvvamp.h) */
  qqvvampprec(int Nf, IRscheme ir);
  // evaluate amplitude coefficients in a reliable way
  /* s = (p1+p2)^2, t = (p1-p3)^2, ma2 = p3^2, mb2 = p4^2 */
  void compute(double s, double t, double ma2, double mb2);
  // output some statistis
  void print_statistics(std::ostream&) const;
 
  double maxrel; // maximal relative target uncertainty
  double maxabs; // maximal absolute target uncertainty
   // best result for amplitude coefficients E (see paper for definition)
  double E[nEcoeff+1][ntype][nloop][nreim];
  std::map<long, long> eventsWithDigits; // counters for some statistics

private:
  qqvvamp<double>    d; // double precision evaluation
  qqvvamp<quadtype>  q; // quad precision evaluation
#ifdef ARBITRARY_PRECISION
  qqvvamp<cln::cl_F> a; // arbitrary precision evaluation
#endif
};
#endif

#endif // QQVVAMPPREC_H
